import type { ValidationModeFlags } from '../types';
declare const _default: (isBlurEvent: boolean, isTouched: boolean, isSubmitted: boolean, reValidateMode: {
    isOnBlur: boolean;
    isOnChange: boolean;
}, mode: Partial<ValidationModeFlags>) => boolean;
export default _default;
//# sourceMappingURL=skipValidation.d.ts.map