import type { DeepPartial, FieldValues, Names } from '../types';
declare const _default: <T>(names: string | string[] | undefined, _names: Names, formValues?: FieldValues, isGlobal?: boolean, defaultValue?: DeepPartial<T> | unknown) => any;
export default _default;
//# sourceMappingURL=generateWatchOutput.d.ts.map