import type { FieldArrayMethodProps, InternalFieldName } from '../types';
declare const _default: (name: InternalFieldName, index: number, options?: FieldArrayMethodProps) => string;
export default _default;
//# sourceMappingURL=getFocusFieldName.d.ts.map