import type { FieldError, Ref, ValidateResult } from '../types';
export default function getValidateError(result: ValidateResult, ref: Ref, type?: string): FieldError | void;
//# sourceMappingURL=getValidateError.d.ts.map