declare const _default: <T extends string | readonly string[] | undefined>(name?: T, signalName?: string, exact?: boolean) => boolean;
export default _default;
//# sourceMappingURL=shouldSubscribeByName.d.ts.map