type CheckboxFieldResult = {
    isValid: boolean;
    value: string | string[] | boolean | undefined;
};
declare const _default: (options?: HTMLInputElement[]) => CheckboxFieldResult;
export default _default;
//# sourceMappingURL=getCheckboxValue.d.ts.map