import type { Field, FieldValues, InternalFieldErrors, InternalNameSet } from '../types';
declare const _default: <T extends FieldValues>(field: Field, disabledFieldNames: InternalNameSet, formValues: T, validateAllFieldCriteria: boolean, shouldUseNativeValidation?: boolean, isFieldArray?: boolean) => Promise<InternalFieldErrors>;
export default _default;
//# sourceMappingURL=validateField.d.ts.map