export declare const isObjectType: (value: unknown) => value is object;
declare const _default: <T extends object>(value: unknown) => value is T;
export default _default;
//# sourceMappingURL=isObject.d.ts.map