import type { ValidationRule, ValidationValue, ValidationValueMessage } from '../types';
declare const _default: <T extends ValidationValue>(rule?: ValidationRule<T> | ValidationValueMessage<T>) => string | T | undefined;
export default _default;
//# sourceMappingURL=getRuleValue.d.ts.map