import type { FieldPath, FieldValues } from '../types';
declare const _default: (object: FieldValues, path: FieldPath<FieldValues>, value?: unknown) => void;
export default _default;
//# sourceMappingURL=set.d.ts.map