import type { ValidationRule } from '../types';
declare const _default: (validationData?: ValidationRule) => {
    value: string | number | boolean | RegExp | undefined;
    message: string;
};
export default _default;
//# sourceMappingURL=getValueAndMessage.d.ts.map