type RadioFieldResult = {
    isValid: boolean;
    value: number | string | null;
};
declare const _default: (options?: HTMLInputElement[]) => RadioFieldResult;
export default _default;
//# sourceMappingURL=getRadioValue.d.ts.map