import type { InternalFieldName } from '../types';
declare const _default: (names: Set<InternalFieldName>, name: InternalFieldName) => boolean;
export default _default;
//# sourceMappingURL=isNameInFieldArray.d.ts.map