import type { InternalFieldName, Names } from '../types';
declare const _default: (name: InternalFieldName, _names: Names, isBlurEvent?: boolean) => boolean;
export default _default;
//# sourceMappingURL=isWatched.d.ts.map