import type { Field, NativeFieldValue } from '../types';
declare const _default: <T extends NativeFieldValue>(value: T, { valueAsNumber, valueAsDate, setValueAs }: Field["_f"]) => any;
export default _default;
//# sourceMappingURL=getFieldValueAs.d.ts.map