import type { FieldError, FieldErrors, FieldValues, InternalFieldName } from '../types';
declare const _default: <T extends FieldValues = FieldValues>(errors: FieldErrors<T>, error: Partial<Record<string, FieldError>>, name: InternalFieldName) => FieldErrors<T>;
export default _default;
//# sourceMappingURL=updateFieldArrayRootError.d.ts.map